@@ -99,8 +99,8 @@ implements the following new ones.
 
 =head2 auto_upgrade
 
-  my $upgrade = $mem->auto_upgrade;
-  $mem        = $mem->auto_upgrade(1);
+  my $bool = $mem->auto_upgrade;
+  $mem     = $mem->auto_upgrade($bool);
 
 Try to detect if content size exceeds C<max_memory_size> limit and
 automatically upgrade to a L<Mojo::Asset::File> object.
