@@ -75,7 +75,7 @@ sub decode {
     # Object
     elsif (m/\G\{/gc) { $ref = _decode_object() }
 
-    # Unexpected
+    # Invalid character
     else { _exception('Expected array or object') }
 
     # Leftover data
@@ -169,7 +169,7 @@ sub _decode_string {
   m!\G((?:(?:[^\x00-\x1f\\"]|\\(?:["\\/bfnrt]|u[0-9a-fA-F]{4})){0,32766})*)!gc;
   my $str = $1;
 
-  # Missing quote
+  # Invalid character
   unless (m/\G"/gc) {
     _exception('Unexpected character or invalid escape while parsing string')
       if m/\G[\x00-\x1f\\]/;
@@ -244,7 +244,7 @@ sub _decode_value {
   # Null
   return undef if m/\Gnull/gc;
 
-  # Invalid data
+  # Invalid character
   _exception('Expected string, array, object, number, boolean or null');
 }
 
@@ -363,13 +363,18 @@ C<Scalar> that has been used in numeric context is considered a number.
   {"foo": "bar"} -> {foo => 'bar'}
 
 Literal names will be translated to and from L<Mojo::JSON> constants or a
-similar native Perl value. In addition C<Scalar> references will be used to
-generate booleans, based on if their values are true or false.
+similar native Perl value.
 
   true  -> Mojo::JSON->true
   false -> Mojo::JSON->false
   null  -> undef
 
+In addition C<Scalar> references will be used to generate booleans, based on
+if their values are true or false.
+
+  \1 -> true
+  \0 -> false
+
 Decoding UTF-16 (LE/BE) and UTF-32 (LE/BE) will be handled transparently,
 encoding will only generate UTF-8. The two Unicode whitespace characters
 C<u2028> and C<u2029> will always be escaped to make JSONP easier.
