@@ -332,7 +332,7 @@ Host part of this URL in punycode format.
 
 =head2 is_abs
 
-  my $success = $url->is_abs;
+  my $bool = $url->is_abs;
 
 Check if URL is absolute.
 
