@@ -30,8 +30,8 @@ sub new {
 
 sub app {
   my ($self, $app) = @_;
-  return $self->ua->app unless $app;
-  $self->ua->app($app);
+  return $self->ua->server->app unless $app;
+  $self->ua->server->app($app);
   return $self;
 }
 
@@ -435,7 +435,7 @@ User agent used for testing, defaults to a L<Mojo::UserAgent> object.
   $t->ua->max_redirects(10);
 
   # Use absolute URL for request with Basic authentication
-  my $url = $t->ua->app_url->userinfo('sri:secr3t')->path('/secrets.json');
+  my $url = $t->ua->server->url->userinfo('sri:secr3t')->path('/secrets.json');
   $t->post_ok($url => json => {limit => 10})
     ->status_is(200)
     ->json_is('/1/content', 'Mojo rocks!');
